@@ -529,8 +529,12 @@ static int deduplicate_and_index_by_id(WayX *wayx,index_t index)
 
     return(1);
    }
+ else
+   {
+    logerror("Way %"Pway_t" is duplicated.\n",wayx->id);
 
- return(0);
+    return(0);
+   }
 }
 
 
@@ -576,7 +580,7 @@ index_t IndexWayX(WaysX *waysx,way_t id)
        if(waysx->idata[mid]<id)      /* Mid point is too low */
           start=mid+1;
        else if(waysx->idata[mid]>id) /* Mid point is too high */
-          end=mid-1;
+          end=mid?(mid-1):mid;
        else                          /* Mid point is correct */
           return(mid);
       }
